package com.jakubu9333.grammarManager.model;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonView;

import lombok.Getter;
import lombok.Setter;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.PostUpdate;
import javax.persistence.SequenceGenerator;

import java.time.LocalDateTime;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Jakub Uhlarik
 */
@Entity
@Getter
@Setter
public class Grammar {

    @Id
    @JsonView(View.All.class)
    @SequenceGenerator(name = "grammar_sequence", allocationSize = 1, sequenceName = "grammar_sequence")
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "grammar_sequence")
    private Long id;

    @JsonView(View.All.class)
    private String uri;

    // @JsonView(View.All.class)
    private LocalDateTime updateTime;


    public Grammar(String uri, Set<Rule> rules, Rule rootRule) {
        this(null, uri, rules, rootRule);
    }

    @JsonCreator
    public Grammar(@JsonProperty("id") Long id, @JsonProperty("uri") String uri, @JsonProperty("rules") Set<Rule> rules, @JsonProperty("rootRule") Rule rootRule) {
        this.rootRule = rootRule;
        this.uri  =uri;
        if (rules == null) {
            this.rules = new HashSet<>();
            return;
        }
        this.rules = rules;
    }

    @OneToMany
    @JsonView(View.RuleWithoutGrammar.class)
    Set<Rule> rules;

    @OneToOne
    @JsonView(View.RuleWithoutGrammar.class)
    Rule rootRule;


    public Grammar(Grammar grammar) {
        this.rootRule = grammar.rootRule;
        this.uri = grammar.getUri();
        this.rules = grammar.getRules();
        this.id = null;
    }

    public Grammar() {
        this(null, null, null);
    }


    @PostUpdate
    public void postUpdate() {
        this.updateTime = LocalDateTime.now();
    }
}
