package com.jakubu9333.grammarManager.model;


import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonView;
import lombok.Getter;
import lombok.Setter;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Jakub Uhlarik
 */
@Entity
@Getter
@Setter
public class Rule {

    @Id
    @JsonView(View.All.class)
    @SequenceGenerator(name = "rule_sequence", allocationSize = 1, sequenceName = "rule_sequence")
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "rule_sequence")
    private Long id;

    @ManyToOne
    @JsonView(View.GrammarWithoutRules.class)
    Grammar grammar;

    @JsonCreator
    public Rule(@JsonProperty("id") Long id, @JsonProperty("grammar") Grammar grammar, @JsonProperty("leftSymbol") String leftSymbol, @JsonProperty("rightSide") Set<RightSidePart> rightSide, @JsonProperty("isRoot") Boolean isRoot) {
        this.root = isRoot != null && isRoot;
        this.grammar = grammar;
        this.leftSymbol = leftSymbol;
        if (rightSide == null) {
            this.rightSide = new HashSet<>();
            return;
        }
        this.rightSide = rightSide;
    }

    public Rule() {
        this(null, null, new HashSet<>(), null);
    }

    public Rule(Grammar grammar, String leftSymbol, Set<RightSidePart> rightSide, Boolean isRoot) {
        this(null, grammar, leftSymbol, rightSide, isRoot);
    }

    @JsonView(View.All.class)
    private String leftSymbol;
    @JsonView(View.All.class)
    @OneToMany
    private Set<RightSidePart> rightSide;

    
    @JsonView(View.All.class)
    private Boolean root;


    public Rule(Rule rule) {
        this.root = false;
        this.id = null;
        this.grammar = rule.getGrammar();
        this.leftSymbol = rule.leftSymbol;
        this.rightSide = rule.getRightSide();
    }


}
